package ru.bitel.bgbilling.modules.inet.dyn.device.manage;

import java.net.InetSocketAddress;
import java.util.List;

import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.modules.inet.access.Access;
import ru.bitel.bgbilling.modules.inet.access.manage.event.InetDeviceManageEvent;
import ru.bitel.bgbilling.modules.inet.api.common.bean.InetServ;
import ru.bitel.bgbilling.modules.inet.dyn.device.snmp.SnmpDeviceManager;
import ru.bitel.bgbilling.modules.inet.dyn.device.terminal.TerminalSession;
import ru.bitel.bgbilling.modules.inet.runtime.InetInterfaceMap;
import ru.bitel.bgbilling.modules.inet.runtime.InetServRuntime;
import ru.bitel.bgbilling.server.util.Setup;
import ru.bitel.common.ParameterMap;
import ru.bitel.common.Utils;
import ru.bitel.oss.systems.inventory.resource.common.bean.Device;
import ru.bitel.oss.systems.inventory.resource.common.bean.DeviceType;

public abstract class AbstractTerminalDeviceManager
	extends SnmpDeviceManager
{
	private static final Logger logger = Logger.getLogger( AbstractTerminalDeviceManager.class );

	protected Access access;
	protected int moduleId;

	protected ParameterMap deviceConfig;

	private TerminalSession terminalSession;

	protected String username;
	protected String password;

	@Override
	public Object init( Setup setup, int moduleId, Device<?, ?> device, DeviceType deviceType, ParameterMap config )
	{
		super.init( setup, moduleId, device, deviceType, config );

		this.moduleId = moduleId;

		try
		{
			// <bean name="access" class="ru.bitel.bgbilling.modules.inet.access.Access" />
			this.access = (Access)Setup.getEnvironment().lookup( "access" );
		}
		catch( NamingException ex )
		{
			logger.error( ex.getMessage(), ex );
		}

		this.deviceConfig = config;

		String terminalProtocol = config.get( "manage.terminal.protocol", null );

		if( Utils.notBlankString( terminalProtocol ) )
		{
			String host = device.getHost();
			int port = 22;

			if( "telnet".equals( terminalProtocol ) )
			{
				port = 23;
			}

			List<InetSocketAddress> hosts = device.getHosts();
			if( hosts.size() > 0 )
			{
				InetSocketAddress socketAddress = hosts.get( 0 );

				host = socketAddress.getAddress().getHostAddress();
				port = socketAddress.getPort();
			}

			host = config.get( "manage.terminal.host", host );
			port = config.getInt( "manage.terminal.port", port );

			terminalSession = TerminalSession.newTerminalSession( terminalProtocol, host, port );

			this.username = device.getUsername();
			this.password = device.getPassword();

			this.username = config.get( "manage.terminal.username", device.getUsername() );
			this.password = config.get( "manage.terminal.password", device.getPassword() );
		}

		return null;
	}

	/**
	 * Получение сервиса (для которого было вызвано событие).
	 * @param e
	 * @return
	 */
	protected InetServ getInetServ( InetDeviceManageEvent e )
	{
		final int servId = e.getServId();

		final InetServRuntime servRuntime = access.getInetServRuntimeMap().get( servId );
		return servRuntime.getInetServ();
	}

	/**
	 * Получение имени интерфейса.
	 * @param e
	 * @param ifaceId
	 * @return
	 * @throws BGException
	 */
	protected String getIfaceTitle( InetDeviceManageEvent e, int ifaceId )
		throws BGException
	{
		final int invDeviceId = e.getDeviceId();

		final InetInterfaceMap ifaceMap = InetInterfaceMap.getInstance( access.moduleId );

		return ifaceMap.getInterfaceTitle( invDeviceId, ifaceId );
	}

	@Override
	public Object destroy()
		throws Exception
	{
		super.destroy();

		terminalSession = null;

		return null;
	}

	@Override
	public Object connect()
		throws Exception
	{
		super.connect();

		if( terminalSession != null )
		{
			terminalSession.connect( username, password );
		}

		return null;
	}

	public TerminalSession getTerminalSession()
		throws Exception
	{
		if( terminalSession != null )
		{
			if( !terminalSession.isConnected() )
			{
				terminalSession.connect( username, password );
			}
		}

		return terminalSession;
	}

	@Override
	public Object disconnect()
		throws Exception
	{
		try
		{
			if( terminalSession != null )
			{
				terminalSession.close();
			}
		}
		finally
		{
			super.disconnect();
		}

		return null;
	}

	/*
	
	@DeviceManagerMethod(title = "Монитор", types = { DeviceManagerMethodType.DEVICE, DeviceManagerMethodType.ACCOUNT })
	public Object monitor()
	{
		return "browse:" + deviceConfig.get( "monitor.url", "http://zabbix.intranet.provider.ru/latest.php?hostid=$monitorHostId" )
									   .replaceAll( "\\$monitorHostId", deviceConfig.get( "monitor.hostId", "" ) );
	}
	
	@DeviceManagerMethod(title = "Telnet")
	public Object telnet()
	{
		return "telnet:" + deviceConfig.get( "telnet.host", deviceConfig.get( "manage.terminal.host", this.host ) ) + " "
			   + deviceConfig.getInt( "telnet.port", deviceConfig.getInt( "manage.terminal.port", 23 ) );
	}
	
	@DeviceManagerMethod(title = "TestMAC", types = { DeviceManagerMethodType.DEVICE })
	public String testMac()
		throws Exception
	{
		final StringBuilder result = new StringBuilder();
	
		snmpClient.walk( SnmpClient.oidFromString( "1.3.6.1.2.1.17.4.3.1.1" ), String.class, ( oid, value ) -> result.append( value ).append( '\n' ) );
	
		return result.toString();
	}
	
	@DeviceManagerMethod(title = "TestPORT", types = { DeviceManagerMethodType.DEVICE })
	public String testPort()
		throws Exception
	{
		final StringBuilder result = new StringBuilder();
	
		snmpClient.walk( SnmpClient.oidFromString( "1.3.6.1.2.1.17.4.3.1.2" ), String.class, ( oid, value ) -> result.append( value ).append( '\n' ) );
	
		return result.toString();
	}
	
	*/
}
